/*******************************************************************
    A telegram bot for your ESP8266 that controls the 
    onboard LED. The LED in this example is active ledoff.

    Parts:
    D1 Mini ESP8266 * - http://s.click.aliexpress.com/e/uzFUnIe
    (or any ESP8266 board)

      = Affilate

    If you find what I do useful and would like to support me,
    please consider becoming a sponsor on Github
    https://github.com/sponsors/witnessmenow/


    Written by Brian Lough
    YouTube: https://www.youtube.com/brianlough
    Tindie: https://www.tindie.com/stores/brianlough/
    Twitter: https://twitter.com/witnessmenow

    Updated by Riccardo Grosso, 06/03/2024
    https://www.ethicaldiy.org
 *******************************************************************/

#include <ESP8266WiFi.h>
#include <WiFiClientSecure.h>
#include <UniversalTelegramBot.h>

// Wifi network station credentials
#define WIFI_SSID "inserisci il nome della tua rete WiFi"
#define WIFI_PASSWORD "inserisci la password della rete WiFi"
// Telegram BOT Token (Get from Botfather)
#define BOT_TOKEN "inserisci il tuo toked (codice alfanumerico lungo)"

const unsigned long BOT_MTBS = 1000; // mean time between scan messages

X509List cert(TELEGRAM_CERTIFICATE_ROOT);
WiFiClientSecure secured_client;
UniversalTelegramBot bot(BOT_TOKEN, secured_client);
unsigned long bot_lasttime; // last time messages' scan has been done

const int ledPin = 0;
int ledStatus = 0;

/* I led e i relè abitualmente si attivano con ledon e si spengono con ledoff
   Ma in alcuni apparecchi commerciali, i comandi sono invertiti
   se lanciando il programma i led/relè si accendono quando si lancia il comando di spegnimento
   e si spengono quando dovrebbero accendersi, allora commentare le due linee seguenti */

 int ledon;   
 int ledoff;
 int changeVal = 1;  

void handleNewMessages(int numNewMessages)
{
  Serial.print("handleNewMessages ");
  Serial.println(numNewMessages);

  for (int i = 0; i < numNewMessages; i++)
  {
    String chat_id = bot.messages[i].chat_id;
    String text = bot.messages[i].text;

    String from_name = bot.messages[i].from_name;
    if (from_name == "")
      from_name = "inserisci il tuo ID (cod. numerico 10 cifre";

    if (text == "/ledon" or text == "/relayon")
    {
      digitalWrite(ledPin, ledon); // turn the LED on 
      ledStatus = 1;
      bot.sendMessage(chat_id, "Il led/relè è attivo", "");
    }

    if (text == "/ledoff" or text == "/relayoff")
    {
      ledStatus = 0;
      digitalWrite(ledPin, ledoff); // turn the LED off 
      bot.sendMessage(chat_id, "Il led/relè è spento", "");
    }

    if (text == "/status")
    {
      if (ledStatus)
      {
        bot.sendMessage(chat_id, "Il led/relè è attivo", "");
      }
      else
      {
        bot.sendMessage(chat_id, "Il led/relè è spento", "");
      }
    }

    if (text == "/start")
    {
      String welcome = "Benvenuto/a " + from_name + ".\n";
      welcome += "This is Flash Led Bot example.\n\n";
      welcome += "/ledon or /relayon: accende il led/relè\n";
      welcome += "/ledoff or /relayoff: spegne il led/relè\n";
      welcome += "/status : mostra lo stato del led/relè\n";
      bot.sendMessage(chat_id, welcome, "Markdown");
    }
  }
}


void setup()
{
  Serial.begin(115200);
  Serial.println();

  if (changeVal == 1){
    ledon = 1;
    ledoff = 0;
  }    
  else{
    ledon = 0;
    ledoff = 1;  
  }

  pinMode(ledPin, OUTPUT); // initialize digital ledPin as an output.
  delay(10);
  digitalWrite(ledPin, ledoff); // initialize pin as off (active ledoff)

  // attempt to connect to Wifi network:
  configTime(0, 0, "pool.ntp.org");      // get UTC time via NTP
  secured_client.setTrustAnchors(&cert); // Add root certificate for api.telegram.org
  Serial.print("Connecting to Wifi SSID ");
  Serial.print(WIFI_SSID);
  WiFi.begin(WIFI_SSID, WIFI_PASSWORD);
  while (WiFi.status() != WL_CONNECTED)
  {
    Serial.print(".");
    delay(500);
  }
  Serial.print("\nWiFi connected. IP address: ");
  Serial.println(WiFi.localIP());

  // Check NTP/Time, usually it is instantaneous and you can delete the code beledoff.
  Serial.print("Retrieving time: ");
  time_t now = time(nullptr);
  while (now < 24 * 3600)
  {
    Serial.print(".");
    delay(100);
    now = time(nullptr);
  }
  Serial.println(now);
}

void loop()
{
  if (millis() - bot_lasttime > BOT_MTBS)
  {
    int numNewMessages = bot.getUpdates(bot.last_message_received + 1);

    while (numNewMessages)
    {
      Serial.println("got response");
      handleNewMessages(numNewMessages);
      numNewMessages = bot.getUpdates(bot.last_message_received + 1);
    }

    bot_lasttime = millis();
  }
}
